@@ -104,8 +104,8 @@ static SEXP ssNewVector(SEXPTYPE type, int vlen)
 	if (type == REALSXP)
 	    REAL(tvec)[j] = ssNA_REAL;
 	else if (type == STRSXP)
-	    STRING(tvec)[j] = STRING(ssNA_STRING)[0];
-    LEVELS(tvec) = 0;
+	    SET_STRING_ELT(tvec, j, STRING_ELT(ssNA_STRING, 0));
+    SETLEVELS(tvec, 0);
     return (tvec);
 }
 /* Global variables needed for the graphics */
@@ -188,31 +188,31 @@ SEXP do_dataentry(SEXP call, SEXP op, SEXP args, SEXP rho)
 	     tvec = CDR(tvec), tvec2 = CDR(tvec2)) {
 	    type = TYPEOF(CAR(tvec)); xmaxused++;
 	    if (CAR(tvec2) != R_NilValue)
-		type = str2type(CHAR(STRING(CAR(tvec2))[0]));
+		type = str2type(CHAR(STRING_ELT(CAR(tvec2), 0)));
 	    if (type != STRSXP)
 		type = REALSXP;
 	    if (CAR(tvec) == R_NilValue) {
 		if (type == NILSXP)
 		    type = REALSXP;
-		CAR(tvec) = ssNewVector(type, 100);
-		TAG(tvec) = install("var1");
-		LEVELS(CAR(tvec)) = 0;
+		SETCAR(tvec, ssNewVector(type, 100));
+		SET_TAG(tvec, install("var1"));
+		SETLEVELS(CAR(tvec), 0);
 	    }
 	    else if (!isVector(CAR(tvec)))
 		errorcall(call, "invalid type for value");
 	    else {
 		if (TYPEOF(CAR(tvec)) != type)
-		    CAR(tvec) = coerceVector(CAR(tvec), type);
-		tmp = LEVELS(CAR(tvec)) = LENGTH(CAR(tvec));
+		    SETCAR(tvec, coerceVector(CAR(tvec), type));
+		tmp = SETLEVELS(CAR(tvec), LENGTH(CAR(tvec)));
 		ymaxused = max(tmp, ymaxused);
 	    }
 	}
     }
     else if (colmodes == R_NilValue ) {
 	PROTECT(inputlist = allocList(1)); nprotect++;
-	CAR(inputlist) = ssNewVector(REALSXP, 100);
-	TAG(inputlist) = install("var1");
-	LEVELS(CAR(inputlist)) = 0;
+	SETCAR(inputlist, ssNewVector(REALSXP, 100));
+	SET_TAG(inputlist, install("var1"));
+	SETLEVELS(CAR(inputlist), 0);
     }
     else {
 	errorcall(call, "invalid parameter(s) ");
@@ -259,14 +259,14 @@ SEXP do_dataentry(SEXP call, SEXP op, SEXP args, SEXP rho)
 		    else
 			REAL(tvec2)[j] = NA_REAL;
 		} else if (TYPEOF(CAR(tvec)) == STRSXP) {
-		    if (!streql(CHAR(STRING(CAR(tvec))[j]),
-				CHAR(STRING(ssNA_STRING)[0])))
-			STRING(tvec2)[j] = STRING(CAR(tvec))[j];
+		    if (!streql(CHAR(STRING_ELT(CAR(tvec), j)),
+				CHAR(STRING_ELT(ssNA_STRING, 0))))
+			SET_STRING_ELT(tvec2, j, STRING_ELT(CAR(tvec), j));
 		    else
-			STRING(tvec2)[j] = NA_STRING;
+			SET_STRING_ELT(tvec2, j, NA_STRING);
 		} else
 		    error("dataentry: internal memory problem");
-	    CAR(tvec) = tvec2;
+	    SETCAR(tvec, tvec2);
 	    UNPROTECT(1);
 	}
     }
@@ -553,7 +553,8 @@ static void printelt(SEXP invec, int vrow, int ssrow, int sscol)
 	}
     }
     else if (TYPEOF(invec) == STRSXP) {
-	if (!streql(CHAR(STRING(invec)[vrow]), CHAR(STRING(ssNA_STRING)[0]))) {
+	if (!streql(CHAR(STRING_ELT(invec, vrow)),
+		    CHAR(STRING_ELT(ssNA_STRING, 0)))) {
 	    strp = EncodeElement(invec, vrow, 0);
 	    printstring(strp, strlen(strp), ssrow, sscol, 0);
 	}
@@ -664,10 +665,10 @@ static SEXP getccol()
 	newcol = 1;
 	xmaxused = wcol;
 	len = max(100, wrow);
-	CAR(tmp) = ssNewVector(REALSXP, len);
+	SETCAR(tmp, ssNewVector(REALSXP, len));
 	if (TAG(tmp) == R_NilValue) {
 	    sprintf(cname, "var%d", wcol);
-	    TAG(tmp) = install(cname);
+	    SET_TAG(tmp, install(cname));
 	}
     }
     if (!isVector(CAR(tmp)))
@@ -682,11 +683,11 @@ static SEXP getccol()
 	    if (type == REALSXP)
 		REAL(tmp2)[i] = REAL(CAR(tmp))[i];
 	    else if (type == STRSXP)
-		STRING(tmp2)[i] = STRING(CAR(tmp))[i];
+		SET_STRING_ELT(tmp2, i, STRING_ELT(CAR(tmp), i));
 	    else
 		error("internal type error in dataentry");
-	LEVELS(tmp2) = LEVELS(CAR(tmp));
-	CAR(tmp) = tmp2;
+	SETLEVELS(tmp2, LEVELS(CAR(tmp)));
+	SETCAR(tmp, tmp2);
     }
     return (tmp);
 }
@@ -713,7 +714,7 @@ static void closerect()
 	c0vec = getccol();
 	cvec = CAR(c0vec);
 	wrow0 = (int)LEVELS(cvec);
-	if (wrow > wrow0) LEVELS(cvec) = wrow;
+	if (wrow > wrow0) SETLEVELS(cvec, wrow);
 	ymaxused = max(ymaxused, wrow);
 	if (clength != 0) {
 	    /* do it this way to ensure NA, Inf, ...  can get set */
@@ -723,21 +724,21 @@ static void closerect()
 	    if (TYPEOF(cvec) == STRSXP) {
 		tvec = allocString(strlen(buf));
 		strcpy(CHAR(tvec), buf);
-		STRING(cvec)[wrow - 1] = tvec;
+		SET_STRING_ELT(cvec, wrow - 1, tvec);
 	    } else
 		REAL(cvec)[wrow - 1] = new;
 	    if (newcol & warn) {
 		/* change mode to character */
 		int levs = LEVELS(cvec);
-		cvec = CAR(c0vec) = coerceVector(cvec, STRSXP);
-		LEVELS(cvec) = levs;
+		cvec = SETCAR(c0vec, coerceVector(cvec, STRSXP));
+		SETLEVELS(cvec, levs);
 		tvec = allocString(strlen(buf));
 		strcpy(CHAR(tvec), buf);
-		STRING(cvec)[wrow - 1] = tvec;
+		SET_STRING_ELT(cvec, wrow - 1, tvec);
 	    }
 	} else {
 	    if (TYPEOF(cvec) == STRSXP)
-		STRING(cvec)[wrow - 1] = NA_STRING;
+		SET_STRING_ELT(cvec, wrow - 1, NA_STRING);
 	    else
 		REAL(cvec)[wrow - 1] = NA_REAL;
 	}
@@ -1065,8 +1066,8 @@ static char *get_cell_text()
 		if (REAL(tvec)[wrow] != ssNA_REAL)
 		    prev = EncodeElement(tvec, wrow, 0);
 	    } else if (TYPEOF(tvec) == STRSXP) {
-		if (!streql(CHAR(STRING(tvec)[wrow]),
-			    CHAR(STRING(ssNA_STRING)[0])))
+		if (!streql(CHAR(STRING_ELT(tvec, wrow)),
+			    CHAR(STRING_ELT(ssNA_STRING, 0))))
 		    prev = EncodeElement(tvec, wrow, 0);
 	    } else error("dataentry: internal memory error");
 	}
@@ -1305,18 +1306,18 @@ static void popupclose(control c)
     }
     tvec = nthcdr(inputlist, popupcol - 1);
     if(ischecked(rb_num) && !isnumeric) {
-	if (CAR(tvec) == R_NilValue) CAR(tvec) = ssNewVector(REALSXP, 100);
+	if (CAR(tvec) == R_NilValue) SETCAR(tvec, ssNewVector(REALSXP, 100));
 	levs = LEVELS(CAR(tvec));
-	CAR(tvec) = coerceVector(CAR(tvec), REALSXP);
-	LEVELS(CAR(tvec)) = levs;
+	SETCAR(tvec, coerceVector(CAR(tvec), REALSXP));
+	SETLEVELS(CAR(tvec), levs);
 
     } else if(ischecked(rb_char) && isnumeric) {
-	if (CAR(tvec) == R_NilValue) CAR(tvec) = ssNewVector(STRSXP, 100);
+	if (CAR(tvec) == R_NilValue) SETCAR(tvec, ssNewVector(STRSXP, 100));
 	levs = LEVELS(CAR(tvec));
-	CAR(tvec) = coerceVector(CAR(tvec), STRSXP);
-	LEVELS(CAR(tvec)) = levs;
+	SETCAR(tvec, coerceVector(CAR(tvec), STRSXP));
+	SETLEVELS(CAR(tvec), levs);
     }
-    TAG(tvec) = install(buf);
+    SET_TAG(tvec, install(buf));
     hide(wconf);
     del(wconf);
 }
@@ -1335,7 +1336,7 @@ static void de_popupmenu(int x_pos, int y_pos, int col)
     blah = get_col_name(popupcol);
     wconf = newwindow("Variable editor",
 		      rect(x_pos + r.x-150, y_pos + r.y-50, 300, 100),
-		      Titlebar | Closebox);
+		      Titlebar | Closebox | Modal);
     setclose(wconf, popupclose);
     setbackground(wconf, bbg);
     lwhat = newlabel("variable name", rect(10, 22, 90, 20), AlignLeft);
@@ -1441,7 +1442,7 @@ static void de_popup_vw()
 
     devw = newwindow("Cell width(s)",
 		      rect(0, 0, 250, 60),
-		      Titlebar | Centered | Closebox);
+		      Titlebar | Centered | Closebox | Modal);
     setclose(devw, vw_close);
     setbackground(devw, bbg);
     lwhat = newlabel("Cell width", rect(10, 20, 70, 20), AlignLeft);
@@ -1522,7 +1523,7 @@ static dataeditor newdataeditor()
     menuitem m;
     
     p = newconsoledata((consolefn) ? consolefn : FixedFont,
-		       pagerrow, pagercol,
+		       pagerrow, pagercol, 0, 0,
 		       consolefg, consoleuser, consolebg,
 		       DATAEDITOR);
     if (!p) return NULL;
@@ -1543,7 +1544,7 @@ static dataeditor newdataeditor()
 #endif
     c = (dataeditor) newwindow(" Data Editor", rect(x, y, w, h),
 			       Document | StandardWindow | TrackMouse |
-			       VScrollbar | HScrollbar | Modal);
+			       VScrollbar | HScrollbar);
     if (!c) {
          freeConsoleData(p);
          return NULL;
