@@ -26,7 +26,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include "graphapp/ga.h"	/* for winmalloc, winfree */
 #include "run.h"
 
 static char RunError[256] = "";
@@ -39,7 +38,7 @@ static char * expandcmd(char *cmd)
     char  fl[MAX_PATH], fn[MAX_PATH];
     int   d , ext;
 
-    if (!(s = (char *) winmalloc(MAX_PATH + strlen(cmd)))) {
+    if (!(s = (char *) malloc(MAX_PATH + strlen(cmd)))) {
 	strcpy(RunError, "Insufficient memory (expandcmd)");
 	return NULL;
     }
@@ -82,7 +81,7 @@ static char * expandcmd(char *cmd)
      }
    }
    if (!d) {
-       winfree(s);
+       free(s);
        strncpy(RunError,p, 200);
        strcat(RunError," not found");
        *q = c;
@@ -126,7 +125,7 @@ static HANDLE pcreate(char* cmd,char *finput,
 	hIN = CreateFile(finput, GENERIC_READ, 0,
 			 &sa, OPEN_EXISTING, 0, NULL);
 	if (hIN == INVALID_HANDLE_VALUE) {
-	    winfree(ecmd);
+	    free(ecmd);
 	    strcpy(RunError, "Impossible to redirect input");
 	    return NULL;
 	}
@@ -174,10 +173,10 @@ static HANDLE pcreate(char* cmd,char *finput,
     if (!ret) {
 	strcpy(RunError, "Impossible to run '");
 	strncat(RunError, ecmd, 200);
-	winfree(ecmd);
+	free(ecmd);
 	return NULL;
     }
-    winfree(ecmd);
+    free(ecmd);
     CloseHandle(pi.hThread);
     return pi.hProcess;
 }
@@ -233,7 +232,7 @@ rpipeOpen(char *cmd, int visible, char *finput)
     HANDLE hOUT, hERR, hThread, hTHIS, hTemp;
     DWORD id;
 
-    if (!(r = (rpipe *) winmalloc(sizeof(struct structRPIPE)))) {
+    if (!(r = (rpipe *) malloc(sizeof(struct structRPIPE)))) {
 	strcpy(RunError, "Insufficient memory (rpipeOpen)");
 	return NULL;
     }
@@ -341,7 +340,7 @@ rpipeClose(rpipe * r)
     CloseHandle(r->write);
     CloseHandle(r->process);
     i = r->exitcode;
-    winfree(r);
+    free(r);
     return i;
 }
 
