@@ -1,5 +1,5 @@
 curve <- function(expr, from, to, n=101, add=FALSE, type="l",
-		  ylab = NULL, log=NULL, ...)
+		  ylab=NULL, log=NULL, xlim=NULL, ...)
 {
     sexpr <- substitute(expr)
     if(is.name(sexpr)) {
@@ -12,20 +12,20 @@ curve <- function(expr, from, to, n=101, add=FALSE, type="l",
 	expr <- sexpr
 	if(is.null(ylab)) ylab <- deparse(sexpr)
     }
-    lims <- delay(par("usr"))
+    lims <- if(is.null(xlim)) delay(par("usr")) else xlim
     if(missing(from)) from <- lims[1]
     if(missing(to)) to <- lims[2]
     lg <- if(length(log)) log else ""
     x <-
 	if(lg != "" && "x" %in% strsplit(lg, NULL)[[1]]) {
-	    rm(list="log",envir=sys.frame(1))# else: warning
+	    ## unneeded now: rm(list="log",envir=sys.frame(1))# else: warning
 	    if(any(c(from,to)<=0))
 		stop("`from' & `to' must be > 0	 with  log=\"x\"")
 	    exp(seq(log(from), log(to), length=n))
 	} else seq(from,to,length=n)
-    y <- eval(expr, envir=list(x = x), enclos=sys.frame(sys.parent(1)))
+    y <- eval(expr, envir=list(x = x), enclos=parent.frame())
     if(add)
 	lines(x, y, type=type, ...)
     else
-	plot(x, y, type=type, ylab = ylab, log=lg, ...)
+	plot(x, y, type=type, ylab = ylab, xlim = xlim, log=lg, ...)
 }
