@@ -1,5 +1,6 @@
-interaction <-
-function(..., drop=FALSE)
+### This is almost like the Primitive ":" for factors
+### (that has no "drop = TRUE") --- it's not used anywhere in "standard R"
+interaction <- function(..., drop=FALSE)
 {
     args <- list(...)
     narg <- length(args)
@@ -15,8 +16,7 @@ function(..., drop=FALSE)
 	    f <- factor(f)
 	l <- levels(f)
 	ans <- ans * length(l) + as.integer(f) - 1
-	if (i == narg) lvs <- l
-	else lvs <- as.vector(outer(l, lvs, paste, sep="."))
+	lvs <- if (i == narg) l	else as.vector(outer(l, lvs, paste, sep="."))
     }
     ans <- ans + 1
     if (drop) {
