@@ -27,7 +27,7 @@ function(x, y = NULL, correct = TRUE, p = rep(1 / length(x), length(x)),
         stop("at least one entry of x must be positive")
 
     if (is.matrix(x)) {
-	METHOD <- "Pearson's Chi-square test"
+	METHOD <- "Pearson's Chi-squared test"
         nr <- nrow(x)
         nc <- ncol(x)
         sr <- apply(x, 1, sum)
@@ -67,7 +67,7 @@ function(x, y = NULL, correct = TRUE, p = rep(1 / length(x), length(x)),
                 YATES <- 0
             STATISTIC <- sum((abs(x - E) - YATES)^2 / E)
             PARAMETER <- (nr - 1) * (nc - 1)
-            PVAL <- 1 - pchisq(STATISTIC, PARAMETER)
+            PVAL <- pchisq(STATISTIC, PARAMETER, lower = FALSE)
         }
     }
     else {
@@ -75,18 +75,18 @@ function(x, y = NULL, correct = TRUE, p = rep(1 / length(x), length(x)),
 	    stop("x must at least have 2 elements")
 	if (length(x) != length(p))
 	    stop("x and p must have the same number of elements")
-	METHOD <- "Chi-square test for given probabilities"
+	METHOD <- "Chi-squared test for given probabilities"
 	E <- n * p
 	names(E) <- names(x)
 	STATISTIC <- sum((x - E) ^ 2 / E)
 	PARAMETER <- length(x) - 1
-        PVAL <- 1 - pchisq(STATISTIC, PARAMETER)
+        PVAL <- pchisq(STATISTIC, PARAMETER, lower = FALSE)
     }
 
     names(STATISTIC) <- "X-squared"
     names(PARAMETER) <- "df"
     if (any(E < 5) && is.finite(PARAMETER))
-	warning("Chi-square approximation may be incorrect")
+	warning("Chi-squared approximation may be incorrect")
 
     structure(list(statistic = STATISTIC,
 		   parameter = PARAMETER,
