@@ -29,7 +29,11 @@
 
 #include "Defn.h"
 /* The next must come after other header files to redefine RE_DUP_MAX */
-#include "regex.h"
+#ifdef USE_SYSTEM_REGEX
+#include <regex.h>
+#else
+#include "Rregex.h"
+#endif
 
 #ifndef MAX
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
@@ -50,7 +54,7 @@ SEXP do_nchar(SEXP call, SEXP op, SEXP args, SEXP env)
     len = LENGTH(x);
     PROTECT(s = allocVector(INTSXP, len));
     for (i = 0; i < len; i++)
-	INTEGER(s)[i] = strlen(CHAR(STRING(x)[i]));
+	INTEGER(s)[i] = strlen(CHAR(STRING_ELT(x, i)));
     if ((d = getAttrib(x, R_DimSymbol)) != R_NilValue)
 	setAttrib(s, R_DimSymbol, d);
     if ((d = getAttrib(x, R_DimNamesSymbol)) != R_NilValue)
@@ -115,7 +119,7 @@ SEXP do_substr(SEXP call, SEXP op, SEXP args, SEXP env)
     for (i = 0; i < len; i++) {
 	start = INTEGER(sa)[i % k];
 	stop = INTEGER(so)[i % l];
-	slen = strlen(CHAR(STRING(x)[i]));
+	slen = strlen(CHAR(STRING_ELT(x, i)));
 	if (start < 1)
 	    start = 1;
 	if (start > stop || start > slen) {
@@ -130,9 +134,9 @@ SEXP do_substr(SEXP call, SEXP op, SEXP args, SEXP env)
 		stop = MAXELTSIZE;
 		warningcall(call, "a string was truncated in substr()");
 		}*/
-	    substr(buff, CHAR(STRING(x)[i]), start, stop);
+	    substr(buff, CHAR(STRING_ELT(x, i)), start, stop);
 	}
-	STRING(s)[i] = mkChar(buff);
+	SET_STRING_ELT(s, i, mkChar(buff));
     }
     UNPROTECT(1);
     AllocBuffer(-1);
@@ -150,6 +154,7 @@ SEXP do_strsplit(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP s, t, tok, x;
     int i, j, len, tlen, ntok;
+    int extended_opt, eflags;
     char *pt = NULL, *split = "", *bufp;
     regex_t reg;
     regmatch_t regmatch[1];
@@ -157,17 +162,24 @@ SEXP do_strsplit(SEXP call, SEXP op, SEXP args, SEXP env)
     checkArity(op, args);
     x = CAR(args);
     tok = CADR(args);
-    if (!isString(x) || !isString(tok))
+    extended_opt = asLogical(CADDR(args));
+
+    if(!isString(x) || !isString(tok))
 	errorcall_return(call,"non-character argument in strsplit()");
+    if(extended_opt == NA_INTEGER) extended_opt = 1;    
+    
+    eflags = 0;
+    if(extended_opt) eflags = eflags | REG_EXTENDED;    
+
     len = LENGTH(x);
     tlen = LENGTH(tok);
     PROTECT(s = allocVector(VECSXP, len));
     for(i = 0; i < len; i++) {
-	AllocBuffer(strlen(CHAR(STRING(x)[i])));
-	strcpy(buff, CHAR(STRING(x)[i]));
+	AllocBuffer(strlen(CHAR(STRING_ELT(x, i))));
+	strcpy(buff, CHAR(STRING_ELT(x, i)));
 	if(tlen > 0) {
 	    /* find out how many splits there will be */
-	    split = CHAR(STRING(tok)[i % tlen]);
+	    split = CHAR(STRING_ELT(tok, i % tlen));
 	    ntok = 0;
 	    /* Careful: need to distinguish empty (rm_eo == 0) from
 	       non-empty (rm_eo > 0) matches.  In the former case, the
@@ -175,10 +187,10 @@ SEXP do_strsplit(SEXP call, SEXP op, SEXP args, SEXP env)
 	       everything before the start of the match, which may be
 	       the empty string (not a ``token'' in the strict sense).
 	       */
-	    if(regcomp(&reg, split, 0))
+	    if(regcomp(&reg, split, eflags))
 		errorcall(call, "invalid split pattern");
 	    bufp = buff;
-	    while(regexec(&reg, bufp, 1, regmatch, 0) == 0) {
+	    while(regexec(&reg, bufp, 1, regmatch, eflags) == 0) {
 		/* Empty matches get the next char, so move by one. */
 		bufp += MAX(regmatch[0].rm_eo, 1);
 		ntok++;
@@ -191,9 +203,9 @@ SEXP do_strsplit(SEXP call, SEXP op, SEXP args, SEXP env)
 		PROTECT(t = allocVector(STRSXP, ntok + 1));
 	    /* and fill with the splits */
 	    bufp = buff;
-	    pt = (char *) realloc(pt, (strlen(buff)+1)*sizeof(char));
+	    pt = (char *) realloc(pt, (strlen(buff)+1) * sizeof(char));
 	    for(j = 0; j < ntok; j++) {
-		regexec(&reg, bufp, 1, regmatch, 0);
+		regexec(&reg, bufp, 1, regmatch, eflags);
 		if(regmatch[0].rm_eo > 0) {
 		    /* Match was non-empty. */
 		    if(regmatch[0].rm_so > 0)
@@ -205,12 +217,13 @@ SEXP do_strsplit(SEXP call, SEXP op, SEXP args, SEXP env)
 		    /* Match was empty. */
 		    pt[0] = *bufp;
 		    pt[1] = '\0';
-		    buff++;
+		    bufp++;
 		}
-		STRING(t)[j] = mkChar(pt);
+		SET_STRING_ELT(t, j, mkChar(pt));
 	    }
 	    if(*bufp != '\0')
-		STRING(t)[ntok] = mkChar(bufp);
+		SET_STRING_ELT(t, ntok, mkChar(bufp));
+	    regfree(&reg);
 	}
 	else {
 	    char bf[2];
@@ -219,14 +232,15 @@ SEXP do_strsplit(SEXP call, SEXP op, SEXP args, SEXP env)
 	    bf[1]='\0';
 	    for (j = 0; j < ntok; j++) {
 		bf[0]=buff[j];
-		STRING(t)[j] = mkChar(bf);
+		SET_STRING_ELT(t, j, mkChar(bf));
 	    }
 	}
 	UNPROTECT(1);
-	VECTOR(s)[i] = t;
+	SET_VECTOR_ELT(s, i, t);
     }
     UNPROTECT(1);
     AllocBuffer(-1);
+    free(pt);
     return s;
 }
 
@@ -360,7 +374,7 @@ SEXP do_abbrev(SEXP call, SEXP op, SEXP args, SEXP env)
     minlen = asInteger(CADR(args));
     uclass = asLogical(CAR(CDDR(args)));
     for (i = 0 ; i < len ; i++)
-	STRING(ans)[i] = stripchars(STRING(CAR(args))[i], minlen);
+	SET_STRING_ELT(ans, i, stripchars(STRING_ELT(CAR(args), i), minlen));
 
     UNPROTECT(1);
     return(ans);
@@ -380,17 +394,17 @@ SEXP do_makenames(SEXP call, SEXP op, SEXP args, SEXP env)
     n = length(arg);
     PROTECT(ans = allocVector(STRSXP, n));
     for (i = 0 ; i < n ; i++) {
-	l = strlen(CHAR(STRING(arg)[i]));
-	if (isalpha(CHAR(STRING(arg)[i])[0])) {
-	    STRING(ans)[i] = allocString(l);
-	    strcpy(CHAR(STRING(ans)[i]), CHAR(STRING(arg)[i]));
+	l = strlen(CHAR(STRING_ELT(arg, i)));
+	if (isalpha(CHAR(STRING_ELT(arg, i))[0])) {
+	    SET_STRING_ELT(ans, i, allocString(l));
+	    strcpy(CHAR(STRING_ELT(ans, i)), CHAR(STRING_ELT(arg, i)));
 	}
 	else {
-	    STRING(ans)[i] = allocString(l + 1);
-	    strcpy(CHAR(STRING(ans)[i]), "X");
-	    strcat(CHAR(STRING(ans)[i]), CHAR(STRING(arg)[i]));
+	    SET_STRING_ELT(ans, i, allocString(l + 1));
+	    strcpy(CHAR(STRING_ELT(ans, i)), "X");
+	    strcat(CHAR(STRING_ELT(ans, i)), CHAR(STRING_ELT(arg, i)));
 	}
-	p = CHAR(STRING(ans)[i]);
+	p = CHAR(STRING_ELT(ans, i));
 	while (*p) {
 	    if (!isalnum(*p) && *p != '.')
 		*p = '.';
@@ -426,14 +440,14 @@ SEXP do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
     if (extended_opt) eflags = eflags | REG_EXTENDED;
     if (igcase_opt) eflags = eflags | REG_ICASE;
 
-    if (regcomp(&reg, CHAR(STRING(pat)[0]), eflags))
+    if (regcomp(&reg, CHAR(STRING_ELT(pat, 0)), eflags))
 	errorcall(call, "invalid regular expression");
 
     n = length(vec);
     ind = allocVector(LGLSXP, n);
     nmatches = 0;
     for (i = 0 ; i < n ; i++) {
-	if (regexec(&reg, CHAR(STRING(vec)[i]), 0, NULL, 0) == 0) {
+	if (regexec(&reg, CHAR(STRING_ELT(vec, i)), 0, NULL, 0) == 0) {
 	    INTEGER(ind)[i] = 1;
 	    nmatches++;
 	}
@@ -446,7 +460,7 @@ SEXP do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
 	j = 0;
 	for (i = 0 ; i < n ; i++)
 	    if (INTEGER(ind)[i]) {
-		STRING(ans)[j++] = STRING(vec)[i];
+		SET_STRING_ELT(ans, j++, STRING_ELT(vec, i));
 		/* FIXME: Want to inherit 'names(vec)': [the following is wrong]
 		   TAG	 (ans)[j]   = TAG(vec)[i]; */
 	    }
@@ -550,7 +564,7 @@ SEXP do_gsub(SEXP call, SEXP op, SEXP args, SEXP env)
     if (extended_opt) eflags = eflags | REG_EXTENDED;
     if (igcase_opt) eflags = eflags | REG_ICASE;
 
-    if (regcomp(&reg, CHAR(STRING(pat)[0]), eflags))
+    if (regcomp(&reg, CHAR(STRING_ELT(pat, 0)), eflags))
 	errorcall(call, "invalid regular expression");
 
     n = length(vec);
@@ -559,8 +573,8 @@ SEXP do_gsub(SEXP call, SEXP op, SEXP args, SEXP env)
     for (i = 0 ; i < n ; i++) {
 	offset = 0;
 	nmatch = 0;
-	s = CHAR(STRING(vec)[i]);
-	t = CHAR(STRING(rep)[0]);
+	s = CHAR(STRING_ELT(vec, i));
+	t = CHAR(STRING_ELT(rep, 0));
 	ns = strlen(s);
 	while (regexec(&reg, &s[offset], 10, regmatch, 0) == 0) {
 	    nmatch += 1;
@@ -574,14 +588,14 @@ SEXP do_gsub(SEXP call, SEXP op, SEXP args, SEXP env)
 		break;
 	}
 	if (nmatch == 0)
-	    STRING(ans)[i] = STRING(vec)[i];
+	    SET_STRING_ELT(ans, i, STRING_ELT(vec, i));
 	else {
-	    STRING(ans)[i] = allocString(ns);
+	    SET_STRING_ELT(ans, i, allocString(ns));
 	    offset = 0;
 	    nmatch = 0;
-	    s = CHAR(STRING(vec)[i]);
-	    t = CHAR(STRING(rep)[0]);
-	    u = CHAR(STRING(ans)[i]);
+	    s = CHAR(STRING_ELT(vec, i));
+	    t = CHAR(STRING_ELT(rep, 0));
+	    u = CHAR(STRING_ELT(ans, i));
 	    ns = strlen(s);
 	    while (regexec(&reg, &s[offset], 10, regmatch, 0) == 0) {
 		for (j = 0; j < regmatch[0].rm_so ; j++)
@@ -627,14 +641,14 @@ SEXP do_regexpr(SEXP call, SEXP op, SEXP args, SEXP env)
 
     eflags = extended_opt ? REG_EXTENDED : 0;
 
-    if (regcomp(&reg, CHAR(STRING(pat)[0]), eflags))
+    if (regcomp(&reg, CHAR(STRING_ELT(pat, 0)), eflags))
 	errorcall(call, "invalid regular expression");
     n = length(text);
     PROTECT(ans = allocVector(INTSXP, n));
     PROTECT(matchlen = allocVector(INTSXP, n));
 
     for (i = 0 ; i < n ; i++) {
-	if(regexec(&reg, CHAR(STRING(text)[i]), 1, regmatch, 0) == 0) {
+	if(regexec(&reg, CHAR(STRING_ELT(text, i)), 1, regmatch, 0) == 0) {
 	    st = regmatch[0].rm_so;
 	    INTEGER(ans)[i] = st + 1; /* index from one */
 	    INTEGER(matchlen)[i] = regmatch[0].rm_eo - st;
@@ -662,12 +676,12 @@ do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
     n = LENGTH(x);
     PROTECT(y = allocVector(STRSXP, n));
     for(i = 0; i < n; i++) {
-	STRING(y)[i] = allocString(strlen(CHAR(STRING(x)[i])));
-	strcpy(CHAR(STRING(y)[i]), CHAR(STRING(x)[i]));
+	SET_STRING_ELT(y, i, allocString(strlen(CHAR(STRING_ELT(x, i)))));
+	strcpy(CHAR(STRING_ELT(y, i)), CHAR(STRING_ELT(x, i)));
     }
 
     for(i = 0; i < n; i++) {
-	for(p = CHAR(STRING(y)[i]); *p != '\0'; p++) {
+	for(p = CHAR(STRING_ELT(y, i)); *p != '\0'; p++) {
 	    *p = tolower(*p);
 	}
     }
@@ -689,12 +703,12 @@ do_toupper(SEXP call, SEXP op, SEXP args, SEXP env)
     n = LENGTH(x);
     PROTECT(y = allocVector(STRSXP, n));
     for(i = 0; i < n; i++) {
-	STRING(y)[i] = allocString(strlen(CHAR(STRING(x)[i])));
-	strcpy(CHAR(STRING(y)[i]), CHAR(STRING(x)[i]));
+	SET_STRING_ELT(y, i, allocString(strlen(CHAR(STRING_ELT(x, i)))));
+	strcpy(CHAR(STRING_ELT(y, i)), CHAR(STRING_ELT(x, i)));
     }
 
     for(i = 0; i < n; i++) {
-	for(p = CHAR(STRING(y)[i]); *p != '\0'; p++) {
+	for(p = CHAR(STRING_ELT(y, i)); *p != '\0'; p++) {
 	    *p = toupper(*p);
 	}
     }
@@ -764,7 +778,7 @@ tr_free_spec(struct tr_spec *trs) {
     }
 }
 
-unsigned char
+static unsigned char
 tr_get_next_char_from_spec(struct tr_spec **p) {
     unsigned char c;
     struct tr_spec *this;
@@ -822,8 +836,8 @@ do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
     trs_new->type = TR_INIT;
     trs_new->next = NULL;
     /* Build the old and new tr_spec lists. */
-    tr_build_spec(CHAR(STRING(old)[0]), trs_old);
-    tr_build_spec(CHAR(STRING(new)[0]), trs_new);
+    tr_build_spec(CHAR(STRING_ELT(old, 0)), trs_old);
+    tr_build_spec(CHAR(STRING_ELT(new, 0)), trs_new);
     /* Initilize the pointers for walking through the old and new
        tr_spec lists and retrieving the next chars from the lists.
        */
@@ -848,12 +862,12 @@ do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
     n = LENGTH(x);
     PROTECT(y = allocVector(STRSXP, n));
     for(i = 0; i < n; i++) {
-	STRING(y)[i] = allocString(strlen(CHAR(STRING(x)[i])));
-	strcpy(CHAR(STRING(y)[i]), CHAR(STRING(x)[i]));
+	SET_STRING_ELT(y, i, allocString(strlen(CHAR(STRING_ELT(x, i)))));
+	strcpy(CHAR(STRING_ELT(y, i)), CHAR(STRING_ELT(x, i)));
     }
 
     for(i = 0; i < length(y); i++) {
-	for(p = CHAR(STRING(y)[i]); *p != '\0'; p++) {
+	for(p = (unsigned char *)CHAR(STRING_ELT(y, i)); *p != '\0'; p++) {
 	    *p = xtable[*p];
 	}
     }
