@@ -53,6 +53,7 @@
 #include "Defn.h"
 #include "Print.h"
 #include "Fileio.h"
+#include "Rconnections.h"
 #include "S.h"
 
 /* Global print parameter struct: */
@@ -77,31 +78,20 @@ void PrintDefaults(SEXP rho)
 
 SEXP do_sink(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    FILE *fp;
     SEXP file;
-    int  append;
+    int  append_, ifile;
+    Rconnection con;
 
     file = CAR(args);
-    append = asLogical(CADR(args));
-    if (append == NA_LOGICAL)
+    append_ = asLogical(CADR(args));
+    if (append_ == NA_LOGICAL)
         errorcall(call, "invalid append specification");
 
-    if(isNull(file)) {
-	if(R_Sinkfile) fclose(R_Sinkfile);
-	R_Sinkfile = NULL;
-	R_Outputfile = R_Consolefile;
-    } else {
-	if (!isString(file) || length(file) != 1)
-	    errorcall(call, "invalid file name");
-	if (append)
-	    fp = R_fopen(R_ExpandFileName(CHAR(STRING(file)[0])), "a");
-	else
-	    fp = R_fopen(R_ExpandFileName(CHAR(STRING(file)[0])), "w");
-	if (!fp)
-	    errorcall(call, "unable to open file");
-	R_Sinkfile = fp;
-	R_Outputfile = fp;
-    }
+    ifile = asInteger(file);
+    con = getConnection(R_SinkCon);
+    switch_stdout(ifile); /* will open new connection if required */
+    if (R_SinkCon >= 3) con->destroy(con);
+    R_SinkCon = R_OutputCon = ifile;
     return R_NilValue;
 }
 
@@ -118,10 +108,12 @@ SEXP do_invisible(SEXP call, SEXP op, SEXP args, SEXP rho)
     }
 }
 
+#if 0
 SEXP do_visibleflag(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     return ScalarLogical(R_Visible);
 }
+#endif
 
 SEXP do_printmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
@@ -208,7 +200,7 @@ SEXP do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho){
     if(!isNull(naprint))  {
 	if(!isString(naprint) || LENGTH(naprint) < 1)
 	    errorcall(call, "invalid na.print specification");
-	R_print.na_string = STRING(naprint)[0];
+	R_print.na_string = STRING_ELT(naprint, 0);
 	R_print.na_width = strlen(CHAR(R_print.na_string));
     }
     args = CDR(args);
@@ -246,7 +238,7 @@ static void PrintGenericVector(SEXP s, SEXP env)
 	PROTECT(dims);
 	PROTECT(t = allocArray(STRSXP, dims));
 	for (i = 0 ; i < ns ; i++) {
-	    switch(TYPEOF(PROTECT(tmp = VECTOR(s)[i]))) {
+	    switch(TYPEOF(PROTECT(tmp = VECTOR_ELT(s, i)))) {
 	    case NILSXP:
 		pbuf = Rsprintf("NULL");
 		break;
@@ -275,7 +267,7 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		break;
 	    }
 	    UNPROTECT(1); /* tmp */
-	    STRING(t)[i] = mkChar(pbuf);
+	    SET_STRING_ELT(t, i, mkChar(pbuf));
 	}
 	if (LENGTH(dims) == 2) {
 	    SEXP rl, cl;
@@ -294,22 +286,22 @@ static void PrintGenericVector(SEXP s, SEXP env)
 	taglen = strlen(tagbuf);
 	ptag = tagbuf + taglen;
 	PROTECT(newcall = allocList(2));
-	CAR(newcall) = install("print");
-	TYPEOF(newcall) = LANGSXP;
+	SETCAR(newcall, install("print"));
+	SET_TYPEOF(newcall, LANGSXP);
 
 	if(ns > 0) {
 	for (i = 0 ; i < ns ; i++) {
 	    if (i > 0) Rprintf("\n");
 	    if (names != R_NilValue &&
-		STRING(names)[i] != R_NilValue &&
-		*CHAR(STRING(names)[i]) != '\0') {
-		if (taglen + strlen(CHAR(STRING(names)[i])) > TAGBUFLEN)
+		STRING_ELT(names, i) != R_NilValue &&
+		*CHAR(STRING_ELT(names, i)) != '\0') {
+		if (taglen + strlen(CHAR(STRING_ELT(names, i))) > TAGBUFLEN)
 		    sprintf(ptag, "$...");
 		else {
-		    if( isValidName(CHAR(STRING(names)[i])) )
-			sprintf(ptag, "$%s", CHAR(STRING(names)[i]));
+		    if( isValidName(CHAR(STRING_ELT(names, i))) )
+			sprintf(ptag, "$%s", CHAR(STRING_ELT(names, i)));
 		    else
-			sprintf(ptag, "$\"%s\"", CHAR(STRING(names)[i]));
+			sprintf(ptag, "$\"%s\"", CHAR(STRING_ELT(names, i)));
 		}
 	    }
 	    else {
@@ -319,11 +311,11 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		    sprintf(ptag, "[[%d]]", i+1);
 	    }
 	    Rprintf("%s\n", tagbuf);
-	    if(isObject(VECTOR(s)[i])) {
-		CADR(newcall) = VECTOR(s)[i];
+	    if(isObject(VECTOR_ELT(s, i))) {
+		SETCADR(newcall, VECTOR_ELT(s, i));
 		eval(newcall, env);
 	    }
-	    else PrintValueRec(VECTOR(s)[i], env);
+	    else PrintValueRec(VECTOR_ELT(s, i), env);
 	    *ptag = '\0';
 	}
 	Rprintf("\n");
@@ -380,7 +372,7 @@ static void printList(SEXP s, SEXP env)
 		pbuf = Rsprintf("?");
 		break;
 	    }
-	    STRING(t)[i++] = mkChar(pbuf);
+	    SET_STRING_ELT(t, i++, mkChar(pbuf));
 	    s = CDR(s);
 	}
 	if (LENGTH(dims) == 2) {
@@ -400,8 +392,8 @@ static void printList(SEXP s, SEXP env)
 	taglen = strlen(tagbuf);
 	ptag = tagbuf + taglen;
 	PROTECT(newcall = allocList(2));
-	CAR(newcall) = install("print");
-	TYPEOF(newcall) = LANGSXP;
+	SETCAR(newcall, install("print"));
+	SET_TYPEOF(newcall, LANGSXP);
 	while (TYPEOF(s) == LISTSXP) {
 	    if (i > 1) Rprintf("\n");
 	    if (TAG(s) != R_NilValue && isSymbol(TAG(s))) {
@@ -422,13 +414,13 @@ static void printList(SEXP s, SEXP env)
 	    }
 	    Rprintf("%s\n", tagbuf);
 	    if(isObject(CAR(s))) {
-		CADR(newcall) = CAR(s);
+		SETCADR(newcall, CAR(s));
 		eval(newcall, env);
 	    }
 	    else PrintValueRec(CAR(s),env);
 	    *ptag = '\0';
 	    s = CDR(s);
-	    i += 1;
+	    i++;
 	}
 	if (s != R_NilValue) {
 	    Rprintf("\n. \n\n");
@@ -448,7 +440,7 @@ static void PrintExpression(SEXP s)
     u = deparse1(s, 0);
     n = LENGTH(u);
     for (i = 0; i < n ; i++) {
-	Rprintf(CHAR(STRING(u)[i]));
+	Rprintf(CHAR(STRING_ELT(u, i)));
 	Rprintf("\n");
     }
 }
@@ -488,7 +480,7 @@ void PrintValueRec(SEXP s,SEXP env)
 	if (isNull(t))
 	    t = deparse1(s, 0);
 	for (i = 0; i < LENGTH(t); i++)
-	    Rprintf("%s\n", CHAR(STRING(t)[i]));
+	    Rprintf("%s\n", CHAR(STRING_ELT(t, i)));
 	if (TYPEOF(s) == CLOSXP) t = CLOENV(s);
 	else t = R_GlobalEnv;
 	if (t != R_GlobalEnv)
@@ -519,14 +511,14 @@ void PrintValueRec(SEXP s,SEXP env)
 	if (TYPEOF(t) == INTSXP) {
 	    if (LENGTH(t) == 1) {
 		PROTECT(t = getAttrib(s, R_DimNamesSymbol));
-		if (t != R_NilValue && VECTOR(t)[0] != R_NilValue) {
+		if (t != R_NilValue && VECTOR_ELT(t, 0) != R_NilValue) {
 		    SEXP nn = getAttrib(t, R_NamesSymbol);
 		    char *title = NULL;
 
 		    if (!isNull(nn))
-		        title = CHAR(STRING(nn)[0]);
+		        title = CHAR(STRING_ELT(nn, 0));
 
-		    printNamedVector(s, VECTOR(t)[0], R_print.quote, title);
+		    printNamedVector(s, VECTOR_ELT(t, 0), R_print.quote, title);
 		}
 		else
 		    printVector(s, 1, R_print.quote);
@@ -555,6 +547,9 @@ void PrintValueRec(SEXP s,SEXP env)
 	}
 	UNPROTECT(1);
 	break;
+    case EXTPTRSXP:
+	Rprintf("<pointer: %p>\n", R_ExternalPtrAddr(s));
+	break;
     default:
 	UNIMPLEMENTED("PrintValueRec");
     }
@@ -564,15 +559,11 @@ void PrintValueRec(SEXP s,SEXP env)
 static void printAttributes(SEXP s,SEXP env)
 {
     SEXP a;
-    SEXP R_LevelsSymbol;
     char *ptag;
-    int i;
 
     a = ATTRIB(s);
     if (a != R_NilValue) {
-	R_LevelsSymbol = install("levels");
 	ptag = tagbuf + strlen(tagbuf);
-	i = 1;
 	while (a != R_NilValue) {
 	    if(isArray(s) || isList(s)) {
 		if(TAG(a) == R_DimSymbol ||
@@ -595,8 +586,6 @@ static void printAttributes(SEXP s,SEXP env)
 	    }
 	    if(TAG(a) == R_CommentSymbol || TAG(a) == R_SourceSymbol)
 		goto nextattr;
-	    if (i > 1)
-		Rprintf("\n");
 	    sprintf(ptag, "attr(,\"%s\")",
 		    EncodeString(CHAR(PRINTNAME(TAG(a))),0,0, Rprt_adj_left));
 	    Rprintf(tagbuf); Rprintf("\n");
@@ -606,7 +595,7 @@ static void printAttributes(SEXP s,SEXP env)
 	    a = CDR(a);
 	}
     }
-}
+}/* printAttributes */
 
 
 /* Print an S-expression using (possibly) local options */
