@@ -24,6 +24,8 @@
  *	------------------------------=#----	Martin Maechler, ETH Zurich
  */
 #include "nmath.h"
+static void I_bessel(double *x, double *alpha, long *nb,
+		     long *ize, double *bi, long *ncalc);
 
 static double exparg = 709.;/* maximal x for UNscaled answer, see below */
 
@@ -54,8 +56,8 @@ double bessel_i(double x, double alpha, double expo)
     return x;
 }
 
-void I_bessel(double *x, double *alpha, long *nb,
-	      long *ize, double *bi, long *ncalc)
+static void I_bessel(double *x, double *alpha, long *nb,
+		     long *ize, double *bi, long *ncalc)
 {
 /* -------------------------------------------------------------------
 
@@ -240,8 +242,6 @@ void I_bessel(double *x, double *alpha, long *nb,
     static double enten = 1e308;
     static double xlarge = 1e4;
 
-    extern double gamma_cody(double);/*--> ./gamma.c */
-
     /* Local variables */
     long nend, intx, nbmx, k, l, n, nstart;
     double pold, test,	p, em, en, empal, emp2al, halfx,
